@@ -27,21 +27,22 @@
 typedef struct _GossipMessagePriv GossipMessagePriv;
 
 struct _GossipMessagePriv {
-	GossipContact     *recipient;
-	gchar             *resource;
+	GossipContact        *recipient;
+	gchar                *resource;
 	
-	GossipContact     *sender;
+	GossipContact        *sender;
 
-	gchar             *body;
-	gchar             *thread;
+	gchar                *subject;
+	gchar                *body;
+	gchar                *thread;
 
-	gossip_time_t      timestamp;
+	gossip_time_t         timestamp;
 
-	gchar             *invite;
+	GossipChatroomInvite *invite;
 	
-	gboolean           request_composing;
+	gboolean              request_composing;
 
-	GossipMessageType  type;
+	GossipMessageType     type;
 };
 
 static void gossip_message_class_init (GossipMessageClass *class);
@@ -62,6 +63,7 @@ enum {
 	PROP_SENDER,
 	PROP_TYPE,
 	PROP_RESOURCE,
+	PROP_SUBJECT,
 	PROP_BODY,
 	PROP_THREAD,
 	PROP_TIMESTAMP,
@@ -94,7 +96,6 @@ gossip_message_get_gtype (void)
 	}
 
 	return type;
-
 }
 
 static void
@@ -142,6 +143,14 @@ gossip_message_class_init (GossipMessageClass *class)
 							      NULL,
 							      G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_SUBJECT,
+					 g_param_spec_string ("subject",
+							      "Subject",
+							      "The message subject",
+							      NULL,
+							      G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_BODY,
 					 g_param_spec_string ("body",
@@ -188,10 +197,16 @@ gossip_message_init (GossipMessage *message)
 	priv->recipient         = NULL;
 	priv->sender            = NULL;
 	priv->resource          = NULL;
+
+	priv->subject           = NULL;
 	priv->body              = NULL;
 	priv->thread            = NULL;
+
 	priv->timestamp         = gossip_time_get_current ();
+
 	priv->request_composing = FALSE;
+
+	priv->invite            = NULL;
 }
 
 static void
@@ -204,15 +219,21 @@ gossip_message_finalize (GObject *object)
 	if (priv->recipient) {
 		g_object_unref (priv->recipient);
 	}
-	g_free (priv->resource);
-	
+
 	if (priv->sender) {
 		g_object_unref (priv->sender);
 	}
 
+	g_free (priv->resource);
+
+	g_free (priv->subject);
 	g_free (priv->body);
 	g_free (priv->thread);
 
+	if (priv->invite) {
+		gossip_chatroom_invite_unref (priv->invite);
+	}
+
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -239,6 +260,9 @@ message_get_property (GObject    *object,
 	case PROP_RESOURCE:
 		g_value_set_string (value, priv->resource);
 		break;
+	case PROP_SUBJECT:
+		g_value_set_string (value, priv->subject);
+		break;
 	case PROP_BODY:
 		g_value_set_string (value, priv->body);
 		break;
@@ -280,6 +304,10 @@ message_set_property (GObject      *object,
 		gossip_message_set_explicit_resource (GOSSIP_MESSAGE (object),
 						      g_value_get_string (value));
 		break;
+	case PROP_SUBJECT:
+		gossip_message_set_subject (GOSSIP_MESSAGE (object),
+					    g_value_get_string (value));
+		break;
 	case PROP_BODY:
 		gossip_message_set_body (GOSSIP_MESSAGE (object),
 					 g_value_get_string (value));
@@ -298,7 +326,8 @@ message_set_property (GObject      *object,
 }
 
 GossipMessage *
-gossip_message_new (GossipMessageType type, GossipContact *recipient)
+gossip_message_new (GossipMessageType  type, 
+		    GossipContact     *recipient)
 {
 	return g_object_new (GOSSIP_TYPE_MESSAGE,
 			     "type", type,
@@ -408,6 +437,37 @@ gossip_message_set_sender (GossipMessage *message, GossipContact *contact)
 	priv->sender = g_object_ref (contact);
 }
 
+const gchar *
+gossip_message_get_subject (GossipMessage *message)
+{
+	GossipMessagePriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), NULL);
+
+	priv = GET_PRIV (message);
+
+	return priv->subject;
+}
+
+void       
+gossip_message_set_subject (GossipMessage *message, 
+			    const gchar   *subject)
+{
+	GossipMessagePriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_MESSAGE (message));
+	
+	priv = GET_PRIV (message);
+
+	g_free (priv->subject);
+
+	if (subject) {
+		priv->subject = g_strdup (subject);
+	} else {
+		priv->subject = NULL;
+	}
+}
+
 const gchar *
 gossip_message_get_body (GossipMessage *message)
 {
@@ -421,7 +481,8 @@ gossip_message_get_body (GossipMessage *message)
 }
 
 void      
-gossip_message_set_body (GossipMessage *message, const gchar *body)
+gossip_message_set_body (GossipMessage *message,
+			 const gchar   *body)
 {
 	GossipMessagePriv *priv;
 
@@ -451,7 +512,8 @@ gossip_message_get_thread (GossipMessage *message)
 }
 
 void       
-gossip_message_set_thread (GossipMessage *message, const gchar *thread)
+gossip_message_set_thread (GossipMessage *message, 
+			   const gchar   *thread)
 {
 	GossipMessagePriv *priv;
 
@@ -481,7 +543,8 @@ gossip_message_get_timestamp (GossipMessage *message)
 }
 
 void
-gossip_message_set_timestamp (GossipMessage *message, gossip_time_t timestamp)
+gossip_message_set_timestamp (GossipMessage *message, 
+			      gossip_time_t  timestamp)
 {
 	GossipMessagePriv *priv;
 	
@@ -497,7 +560,7 @@ gossip_message_set_timestamp (GossipMessage *message, gossip_time_t timestamp)
 	}
 }
 
-const gchar *
+GossipChatroomInvite *
 gossip_message_get_invite (GossipMessage *message)
 {
 	GossipMessagePriv *priv;
@@ -510,7 +573,8 @@ gossip_message_get_invite (GossipMessage *message)
 }
 
 void       
-gossip_message_set_invite (GossipMessage *message, const gchar *invite)
+gossip_message_set_invite (GossipMessage        *message, 
+			   GossipChatroomInvite *invite)
 {
 	GossipMessagePriv *priv;
 
@@ -518,10 +582,12 @@ gossip_message_set_invite (GossipMessage *message, const gchar *invite)
 	
 	priv = GET_PRIV (message);
 
-	g_free (priv->invite);
+	if (priv->invite) {
+		gossip_chatroom_invite_unref (priv->invite);
+	}
 
 	if (invite) {
-		priv->invite = g_strdup (invite);
+		priv->invite = gossip_chatroom_invite_ref (invite);
 	} else {
 		priv->invite = NULL;
 	}
