@@ -23,7 +23,8 @@
 #include <glib/gi18n.h>
 
 /* FIXME: we should really have a definition in config.h so we can
- * include certain protocol head files */
+ * include certain protocol head files.
+ */
 #include <gossip-jabber.h>
 
 #include "libgossip-marshal.h"
@@ -40,9 +41,9 @@ enum {
 	CONTACT_UPDATED,
 	CONTACT_PRESENCE_UPDATED,
 	CONTACT_REMOVED,
-	COMPOSING_EVENT,
+	COMPOSING,
 
-	/* use to get password from user */
+	/* Used to get password from user. */
 	GET_PASSWORD,
 	
 	SUBSCRIPTION_REQUEST,
@@ -92,9 +93,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 
 	/* Maybe include a reason for disconnect? */
 	signals[LOGGED_OUT] = 
@@ -103,9 +104,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 
 	signals[NEW_MESSAGE] = 
 		g_signal_new ("new-message",
@@ -113,9 +114,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_MESSAGE);
 
 	signals[CONTACT_ADDED] = 
 		g_signal_new ("contact-added",
@@ -123,9 +124,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_UPDATED] = 
 		g_signal_new ("contact-updated",
@@ -133,9 +134,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 	
 	signals[CONTACT_PRESENCE_UPDATED] =
 		g_signal_new ("contact-presence-updated",
@@ -143,9 +144,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[CONTACT_REMOVED] = 
 		g_signal_new ("contact-removed",
@@ -153,19 +154,19 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER,
+			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
-	signals[COMPOSING_EVENT] = 
-		g_signal_new ("composing-event",
+	signals[COMPOSING] = 
+		g_signal_new ("composing",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_BOOLEAN,
+			      libgossip_marshal_VOID__OBJECT_BOOLEAN,
 			      G_TYPE_NONE,
-			      2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
+			      2, GOSSIP_TYPE_CONTACT, G_TYPE_BOOLEAN);
 
 	signals[GET_PASSWORD] =
 		g_signal_new ("get-password",
@@ -173,9 +174,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_STRING__POINTER,
+			      libgossip_marshal_STRING__OBJECT,
 			      G_TYPE_STRING,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_ACCOUNT);
 
         signals[SUBSCRIPTION_REQUEST] =
                 g_signal_new ("subscription-request",
@@ -183,9 +184,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-                              libgossip_marshal_VOID__POINTER,
+                              libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
+			      1, GOSSIP_TYPE_CONTACT);
 
 	signals[ERROR] = 
 		g_signal_new ("error",
@@ -193,9 +194,9 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__POINTER_POINTER,
+			      libgossip_marshal_VOID__OBJECT_POINTER,
 			      G_TYPE_NONE,
-			      2, G_TYPE_POINTER, G_TYPE_POINTER);
+			      2, GOSSIP_TYPE_ACCOUNT, G_TYPE_POINTER);
 
 }
 
@@ -669,7 +670,7 @@ gossip_protocol_register_cancel (GossipProtocol *protocol)
 const gchar *
 gossip_protocol_error_to_string (GossipProtocolError error)
 {
-	const gchar *str = _("An unknown error ocurred.");
+	const gchar *str = _("An unknown error occurred.");
 
 	switch (error) {
 	case GOSSIP_PROTOCOL_NO_CONNECTION:
@@ -687,6 +688,9 @@ gossip_protocol_error_to_string (GossipProtocolError error)
 	case GOSSIP_PROTOCOL_DUPLICATE_USER:
 		str = _("The username you are trying already exists.");
 		break;
+	case GOSSIP_PROTOCOL_INVALID_USER:
+		str = _("The username you are trying is not valid.");
+		break;
 	case GOSSIP_PROTOCOL_UNAVAILABLE:
 		str = _("This feature is unavailable.");
 		break;
@@ -694,7 +698,7 @@ gossip_protocol_error_to_string (GossipProtocolError error)
 		str = _("This feature is unauthorized.");
 		break;
 	case GOSSIP_PROTOCOL_SPECIFIC_ERROR:
-		str = _("A specific protocol error occured that was unexpected.");
+		str = _("A specific protocol error occurred that was unexpected.");
 		break;
 	}
 
