@@ -21,6 +21,7 @@
 
 #include <config.h>
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glade/glade.h>
@@ -46,7 +47,7 @@
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowPriv))
 
 #define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
 
 static void       gossip_chat_window_class_init         (GossipChatWindowClass *klass);
 static void       gossip_chat_window_init               (GossipChatWindow      *window);
@@ -58,21 +59,25 @@ static void       chat_window_accel_cb                  (GtkAccelGroup         *
 							 guint                  key,
 							 GdkModifierType        mod,
 							 GossipChatWindow      *window);
-static gboolean   chat_window_status_enter_notify_cb    (GtkWidget             *widget,
+static gboolean   chat_window_tab_enter_notify_cb       (GtkWidget             *widget,
 							 GdkEventCrossing      *event,
 							 GossipChat            *chat);
-static gboolean   chat_window_status_leave_notify_cb    (GtkWidget             *widget,
+static gboolean   chat_window_tab_leave_notify_cb       (GtkWidget             *widget,
 							 GdkEventCrossing      *event,
 							 GossipChat            *chat);
-static gboolean   chat_window_status_button_press_cb    (GtkWidget             *widget,
-							 GdkEventButton        *event,
+static void       chat_window_close_clicked_cb          (GtkWidget             *button,
 							 GossipChat            *chat);
 static GtkWidget *chat_window_create_label              (GossipChatWindow      *window,
 							 GossipChat            *chat);
 static void       chat_window_update_status             (GossipChatWindow      *window,
 							 GossipChat            *chat);
-static void       chat_window_update_title              (GossipChatWindow      *window);
+static void       chat_window_update_title              (GossipChatWindow      *window,
+							 GossipChat            *chat);
 static void       chat_window_update_menu               (GossipChatWindow      *window);
+static gboolean   chat_window_save_geometry_timeout_cb  (GossipChatWindow      *window);
+static gboolean   chat_window_configure_event_cb        (GtkWidget             *widget,
+							 GdkEventConfigure     *event,
+							 GossipChatWindow      *window);
 static void       chat_window_conv_activate_cb          (GtkWidget             *menuitem,
 							 GossipChatWindow      *window);
 static void       chat_window_clear_activate_cb         (GtkWidget             *menuitem,
@@ -157,7 +162,9 @@ struct _GossipChatWindowPriv {
 	GList       *chats;
 	GList       *chats_new_msg;
 	GList       *chats_composing;
+
 	GossipChat  *current_chat;
+
 	gboolean     new_msg;
 
 	GtkWidget   *dialog;
@@ -188,6 +195,13 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *menu_tabs_left;
 	GtkWidget   *menu_tabs_right;
 	GtkWidget   *menu_tabs_detach;
+
+	/* Geometry information */
+	guint        save_geometry_id;
+	gint         last_x;
+	gint         last_y;
+	gint         last_w;
+	gint         last_h;
 };
 
 static GList *chat_windows = NULL;
@@ -215,6 +229,16 @@ gossip_chat_window_class_init (GossipChatWindowClass *klass)
 	object_class->finalize = gossip_chat_window_finalize;
 
 	g_type_class_add_private (object_class, sizeof (GossipChatWindowPriv));
+
+	/* Set up a style for the close button with no focus padding. */ 	 
+	gtk_rc_parse_string ( 	 
+		"style \"gossip-close-button-style\"\n" 	 
+		"{\n" 	 
+		"  GtkWidget::focus-padding = 0\n" 	 
+		"  xthickness = 0\n" 	 
+		"  ythickness = 0\n" 	 
+		"}\n" 	 
+		"widget \"*.gossip-close-button\" style \"gossip-close-button-style\"");
 }
 
 static void
@@ -260,6 +284,7 @@ gossip_chat_window_init (GossipChatWindow *window)
 
 	gossip_glade_connect (glade, 
 			      window,
+			      "chat_window", "configure-event", chat_window_configure_event_cb,
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_log", "activate", chat_window_log_activate_cb,
@@ -449,6 +474,11 @@ gossip_chat_window_finalize (GObject *object)
 					      chat_window_disconnected_cb,
 					      window);
 
+	if (priv->save_geometry_id != 0) {
+		g_source_remove (priv->save_geometry_id);
+		priv->save_geometry_id = 0;
+	}
+
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
 
@@ -505,79 +535,140 @@ chat_window_accel_cb (GtkAccelGroup    *accelgroup,
 }
 
 static gboolean
-chat_window_status_enter_notify_cb (GtkWidget        *widget,
-				    GdkEventCrossing *event,
-				    GossipChat       *chat)
+chat_window_tab_enter_notify_cb (GtkWidget        *widget,
+				 GdkEventCrossing *event,
+				 GossipChat       *chat)
 {
-	GossipChatWindow     *window;
-	GossipChatWindowPriv *priv;
-	GtkImage             *image;
-	GdkPixbuf            *pixbuf;
+	GtkButton *button;
 
-	pixbuf = gossip_pixbuf_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	image = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
-	gtk_image_set_from_pixbuf (image, pixbuf);
-
-	/* Set tooltip */
-	window = gossip_chat_get_window (chat);
-	priv = GET_PRIV (window);
-
-	gtk_tooltips_set_tip (priv->tooltips,
-			      widget,
-			      _("Close this chat window"),
-			      NULL);
-
-	/* Allow closing with left button click */
- 	g_signal_connect (widget, "button_press_event", 
- 			  G_CALLBACK (chat_window_status_button_press_cb), 
- 			  chat); 
+	button = g_object_get_data (G_OBJECT (chat), "chat-window-tab-button");
+	gtk_widget_show (GTK_WIDGET (button));
 
 	return FALSE;
 }
 
 static gboolean
-chat_window_status_leave_notify_cb (GtkWidget        *widget,
-				    GdkEventCrossing *event,
-				    GossipChat       *chat)
+chat_window_tab_leave_notify_cb (GtkWidget        *widget,
+				 GdkEventCrossing *event,
+				 GossipChat       *chat)
 {
-	GossipChatWindow *window;
-	GtkImage         *image;
-	GdkPixbuf        *pixbuf;
+	GtkButton *button;
+	gint       w, h;
 
-	image = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
+	gtk_window_get_size (GTK_WINDOW (widget), &w, &h);
 
-	window = gossip_chat_get_window (chat);
-	pixbuf = chat_window_get_status_pixbuf (window, chat);
-	gtk_image_set_from_pixbuf (image, pixbuf);
-	
-	/* Stop the close window left mouse click ability */
- 	g_signal_handlers_disconnect_by_func (widget,
-					      chat_window_status_button_press_cb, 
-					      chat); 
-	return FALSE;
-}
-
-static gboolean
-chat_window_status_button_press_cb (GtkWidget      *widget,
-				    GdkEventButton *event,
-				    GossipChat     *chat)
-{
-	GossipChatWindow *window;
-
-	if (event->button != 1) {
-		return FALSE;
+	if (event->x < 0 || event->y < 0 || 
+	    event->x > w || event->y > h) {
+		button = g_object_get_data (G_OBJECT (chat), "chat-window-tab-button");
+		gtk_widget_hide (GTK_WIDGET (button));
 	}
 
+	return FALSE;
+}
+
+static void
+chat_window_close_clicked_cb (GtkWidget  *button,
+			      GossipChat *chat)
+{
+	GossipChatWindow *window;
+
 	window = gossip_chat_get_window (chat);
 	gossip_chat_window_remove_chat (window, chat);
-
-	return FALSE;
 }
 
 static GtkWidget *
 chat_window_create_label (GossipChatWindow *window,
 			  GossipChat       *chat)
 {
+	GossipChatWindowPriv *priv;
+	GtkWidget            *hbox;
+	GtkWidget            *name_label;
+	GtkWidget            *status_image;
+	GtkWidget            *close_button;
+	GtkWidget            *close_image;
+	const gchar          *name;
+	GtkWidget            *event_box; 
+	GtkWidget            *event_box_hbox; 
+	PangoAttrList        *attr_list;
+	PangoAttribute       *attr;
+	GtkRequisition        size;
+
+	priv = GET_PRIV (window);
+	
+	hbox = gtk_hbox_new (FALSE, 2);
+
+ 	event_box = gtk_event_box_new (); 
+ 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event_box), FALSE); 
+
+	name = gossip_chat_get_name (chat);
+	name_label = gtk_label_new (name);
+
+	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
+	
+	attr_list = pango_attr_list_new ();
+	attr = pango_attr_scale_new (1/1.2);
+	attr->start_index = 0;
+	attr->end_index = -1;
+	pango_attr_list_insert (attr_list, attr);
+	gtk_label_set_attributes (GTK_LABEL (name_label), attr_list);
+	pango_attr_list_unref (attr_list);
+
+	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
+	g_object_set_data (G_OBJECT (chat), "chat-window-tab-label", name_label); 
+
+	status_image = gtk_image_new ();
+
+	event_box_hbox = gtk_hbox_new (FALSE, 2);
+
+	gtk_box_pack_start (GTK_BOX (event_box_hbox), status_image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (event_box_hbox), name_label, TRUE, TRUE, 0);
+
+	g_object_set_data (G_OBJECT (chat), "chat-window-tab-image", status_image);
+  	g_object_set_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget", event_box);
+
+	chat_window_update_tooltip (window, chat);
+
+	close_button = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+
+	/* We don't want focus/keynav for the button to avoid clutter, and
+	 * Ctrl-W works anyway.
+	 */
+	GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_FOCUS);
+	GTK_WIDGET_UNSET_FLAGS (close_button, GTK_CAN_DEFAULT);
+
+	/* Set the name to make the special rc style match. */
+	gtk_widget_set_name (close_button, "gossip-close-button");
+	g_object_set_data (G_OBJECT (chat), "chat-window-tab-button", close_button);
+
+	close_image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+
+	gtk_widget_size_request (close_image, &size);
+	gtk_widget_set_size_request (close_button, size.width, size.height);
+	
+	gtk_container_add (GTK_CONTAINER (close_button), close_image);
+	gtk_container_set_border_width (GTK_CONTAINER (close_button), 0);
+
+	gtk_container_add (GTK_CONTAINER (event_box), event_box_hbox);
+	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
+
+	g_signal_connect (close_button,
+			  "clicked",
+			  G_CALLBACK (chat_window_close_clicked_cb),
+			  chat);
+
+	/* Set up tooltip */
+	chat_window_update_tooltip (window, chat);
+
+	gtk_widget_show_all (hbox);
+
+	/* We do not show this to begin with. */
+	gtk_widget_hide (close_button);
+
+	return hbox;
+
+#if 0
 	GossipChatWindowPriv *priv;
 	GtkWidget            *hbox;
 	GtkWidget            *label;
@@ -640,7 +731,7 @@ chat_window_create_label (GossipChatWindow *window,
 			  G_CALLBACK (chat_window_status_enter_notify_cb),
 			  chat);
 	g_signal_connect (event_box_image, "leave-notify-event",
-			  G_CALLBACK (chat_window_status_leave_notify_cb),
+			  G_CALLBACK (chat_window_tab_leave_notify_cb),
 			  chat);
 
 	/* Set up tooltip */
@@ -650,6 +741,7 @@ chat_window_create_label (GossipChatWindow *window,
 	gtk_widget_show_all (event_box_tab);
 
 	return event_box_tab;
+#endif
 }
 
 static void
@@ -667,18 +759,30 @@ chat_window_update_status (GossipChatWindow *window,
 }
 
 static void
-chat_window_update_title (GossipChatWindow *window)
+chat_window_update_title (GossipChatWindow *window,
+			  GossipChat       *chat)
 {
 	GossipChatWindowPriv *priv;
+	const gchar          *name;
 	gchar                *title; 
 	GdkPixbuf 	     *pixbuf;
 
 	priv = GET_PRIV (window);
 
+	/* Use the name from the supplied chat, which happens when we get a
+	 * message from an tab other than the active one. This makes it easier
+	 * to see who talked to you when you have multiple tabs.
+	 */
+	if (chat) {
+		name = gossip_chat_get_name (chat);
+	} else {
+		name = gossip_chat_get_name (priv->current_chat);
+	}
+
 	title = g_strdup_printf (("%s%s"),
 				 priv->new_msg ? "*" : "",
-				 gossip_chat_get_name (priv->current_chat));
-
+				 name);
+	
 	gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
 
 	if (priv->new_msg) {
@@ -728,7 +832,7 @@ chat_window_update_menu (GossipChatWindow *window)
 	gtk_widget_set_sensitive (priv->menu_tabs_right, !last_page);
 	gtk_widget_set_sensitive (priv->menu_conv_info, contact != NULL);
 
-	if (gossip_chat_get_group_chat (priv->current_chat)) {
+	if (gossip_chat_is_group_chat (priv->current_chat)) {
 		GossipGroupChat       *group_chat;
 		GossipChatroomManager *manager;
 		GossipChatroomId       id;
@@ -862,7 +966,7 @@ chat_window_log_activate_cb (GtkWidget        *menuitem,
 
 	priv = GET_PRIV (window);
 
-	if (gossip_chat_get_group_chat (priv->current_chat)) {
+	if (gossip_chat_is_group_chat (priv->current_chat)) {
 		GossipGroupChat        *group_chat;
 		GossipChatroomProvider *provider;
 		GossipChatroom         *chatroom;
@@ -897,6 +1001,54 @@ chat_window_info_activate_cb (GtkWidget        *menuitem,
 	gossip_contact_info_dialog_show (contact);
 }
 
+static gboolean
+chat_window_save_geometry_timeout_cb (GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+
+	priv = GET_PRIV (window);
+
+	gossip_chat_save_geometry (priv->current_chat, 
+				   priv->last_x, priv->last_y, 
+				   priv->last_w, priv->last_h);
+
+	priv->save_geometry_id = 0;
+
+	return FALSE;
+}
+
+static gboolean 
+chat_window_configure_event_cb (GtkWidget         *widget,
+				GdkEventConfigure *event,
+				GossipChatWindow  *window)
+{
+	GossipChatWindowPriv *priv;
+
+	priv = GET_PRIV (window);
+
+	/* Only save geometry information if there is ONE chat visible */
+	if (g_list_length (priv->chats) > 1) {
+		return FALSE;
+	}
+
+	if (priv->save_geometry_id != 0) {
+		g_source_remove (priv->save_geometry_id);
+		priv->save_geometry_id = 0;
+	}
+
+	priv->save_geometry_id = 
+		g_timeout_add (500,
+			       (GSourceFunc) chat_window_save_geometry_timeout_cb,
+			       window);
+
+	priv->last_x = event->x;
+	priv->last_y = event->y;
+	priv->last_w = event->width;
+	priv->last_h = event->height;
+
+	return FALSE;
+}
+
 static void
 chat_window_conv_activate_cb (GtkWidget        *menuitem,
 			      GossipChatWindow *window)
@@ -906,7 +1058,7 @@ chat_window_conv_activate_cb (GtkWidget        *menuitem,
 
 	priv = GET_PRIV (window);
 	
-	if (gossip_chat_get_group_chat (priv->current_chat)) {
+	if (gossip_chat_is_group_chat (priv->current_chat)) {
 		GossipGroupChat        *group_chat;
 		GossipChatroomProvider *provider;
 		GossipChatroom         *chatroom;
@@ -976,7 +1128,7 @@ chat_window_add_room_activate_cb (GtkWidget        *menuitem,
 
 	g_return_if_fail (priv->current_chat != NULL);
 
-	if (!gossip_chat_get_group_chat (priv->current_chat)) {
+	if (!gossip_chat_is_group_chat (priv->current_chat)) {
 		return;
 	}
 
@@ -1071,7 +1223,7 @@ chat_window_paste_activate_cb (GtkWidget        *menuitem,
 
 static void
 chat_window_tabs_left_activate_cb (GtkWidget        *menuitem,
-				  GossipChatWindow *window)
+				   GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	GossipChat           *chat;
@@ -1090,14 +1242,14 @@ chat_window_tabs_left_activate_cb (GtkWidget        *menuitem,
 				   gossip_chat_get_widget (chat),
 				   index - 1);
 
-	chat_window_update_title (window);
+	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
 
 static void
 chat_window_tabs_right_activate_cb (GtkWidget        *menuitem,
-				   GossipChatWindow *window)
+				    GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	GossipChat           *chat;
@@ -1113,7 +1265,7 @@ chat_window_tabs_right_activate_cb (GtkWidget        *menuitem,
 				   gossip_chat_get_widget (chat),
 				   index + 1);
 
-	chat_window_update_title (window);
+	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
@@ -1240,11 +1392,11 @@ chat_window_new_message_cb (GossipChat       *chat,
 		GossipContact *own_contact;
 
 		priv->new_msg = TRUE;
-		chat_window_update_title (window);
+		chat_window_update_title (window, chat);
 
-		own_contact = gossip_chat_get_own_contact (chat);
+		if (gossip_chat_is_group_chat (chat)) {
+			own_contact = gossip_chat_get_own_contact (chat);
 
-		if (gossip_chat_get_group_chat (chat)) {
 			DEBUG_MSG (("ChatWindow: Should we highlight this nick?"));
 			if (gossip_chat_should_highlight_nick (message, own_contact)) {
 				gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), TRUE);
@@ -1313,7 +1465,7 @@ chat_window_switch_page_cb (GtkNotebook	     *notebook,
 	priv->current_chat = chat;
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
 
-	chat_window_update_title (window);
+	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 	chat_window_update_status (window, chat);
 }
@@ -1326,6 +1478,7 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 	GossipChatWindowPriv *priv;
 	GossipChat           *chat;
 	GtkWidget            *label;
+	gboolean              expand = TRUE;
 
  	DEBUG_MSG (("ChatWindow: Tab added"));
 
@@ -1341,7 +1494,21 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), child, label);
 
 	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), child,
-					    TRUE, TRUE, GTK_PACK_START);
+					    expand, TRUE, GTK_PACK_START);
+
+	/* Set up motion events to show/hide the close button when the
+	 * pointer enters the tab area
+	 */
+	gtk_widget_add_events (GTK_WIDGET (priv->dialog), 
+			       GDK_ENTER_NOTIFY_MASK |
+			       GDK_LEAVE_NOTIFY_MASK);
+	g_signal_connect (priv->dialog, "enter-notify-event",
+			  G_CALLBACK (chat_window_tab_enter_notify_cb),
+			  chat);
+	g_signal_connect (priv->dialog, "leave-notify-event",
+			  G_CALLBACK (chat_window_tab_leave_notify_cb),
+			  chat);
+
 
 	g_signal_connect (chat, "status_changed",
 			  G_CALLBACK (chat_window_status_changed_cb),
@@ -1357,7 +1524,7 @@ chat_window_tab_added_cb (GossipNotebook   *notebook,
 			  window);
 
 	chat_window_update_status (window, chat);
-	chat_window_update_title (window);
+	chat_window_update_title (window, NULL);
 	chat_window_update_menu (window);
 
 	gossip_chat_scroll_down (chat);
@@ -1379,6 +1546,14 @@ chat_window_tab_removed_cb (GossipNotebook   *notebook,
 	
 	gossip_chat_set_window (chat, NULL);
 
+	g_signal_handlers_disconnect_by_func (priv->dialog,
+					      G_CALLBACK (chat_window_tab_enter_notify_cb),
+					      chat);
+
+	g_signal_handlers_disconnect_by_func (priv->dialog,
+					      G_CALLBACK (chat_window_tab_leave_notify_cb),
+					      chat);
+
 	g_signal_handlers_disconnect_by_func (chat,
 					      G_CALLBACK (chat_window_status_changed_cb),
 					      window);
@@ -1424,11 +1599,11 @@ chat_window_tab_detached_cb (GossipNotebook   *notebook,
 				 &y,
 				 NULL);
 	
-	gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
-	
 	gossip_chat_window_remove_chat (window, chat);
 	gossip_chat_window_add_chat (new_window, chat);
 
+	gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
+
 	gtk_widget_show (priv->dialog);
 }
 
@@ -1452,7 +1627,7 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 
 	priv->new_msg = FALSE;
 
-	chat_window_update_title (window);
+	chat_window_update_title (window, NULL);
 
 	return FALSE;
 }
@@ -1551,11 +1726,23 @@ gossip_chat_window_add_chat (GossipChatWindow *window,
 	label = chat_window_create_label (window, chat);
 
 	if (g_list_length (priv->chats) == 0) {
-		gint width, height;
+		gint x, y, w, h;
 
-		/* First chat, resize the window to its preferred size. */
-		gossip_chat_get_geometry (chat, &width, &height);
-		gtk_window_resize (GTK_WINDOW (priv->dialog), width, height);
+		gossip_chat_load_geometry (chat, &x, &y, &w, &h);
+		
+		if (x >= 1 && y >= 1) {
+			/* Let the window manager position it if we
+			 * don't have good x, y coordinates.  
+			 */
+			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
+		}
+
+		if (w >= 1 && h >= 1) {
+			/* Use the defaults from the glade file if we
+			 * don't have good w, h geometry.  
+			 */
+			gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
+		}
 	}
 
 	gossip_notebook_insert_page (GOSSIP_NOTEBOOK (priv->notebook),
@@ -1613,4 +1800,3 @@ gossip_chat_window_has_focus (GossipChatWindow *window)
 
 	return has_focus;
 }
-
