@@ -20,26 +20,37 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <stdlib.h>
-#include <config.h>
+
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <glib/gi18n.h>
 
+#include "gossip-app.h"
+#include "gossip-chat.h"
 #include "gossip-chat-window.h"
+#include "gossip-geometry.h"
 #include "gossip-marshal.h"
+#include "gossip-preferences.h"
 #include "gossip-spell.h"
 #include "gossip-spell-dialog.h"
 #include "gossip-ui-utils.h"
-#include "gossip-preferences.h"
-#include "gossip-chat.h"
-#include "gossip-app.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT, GossipChatPriv))
 
-#define DEBUG_MSG(x)  
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+#define DEBUG_MSG(x)
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");   */
+
+#define CHAT_DIR_CREATE_MODE    (S_IRUSR | S_IWUSR | S_IXUSR)
+#define CHAT_FILE_CREATE_MODE   (S_IRUSR | S_IWUSR)
+
+#define CHAT_KEY_FILENAME       "geometry.ini"
+#define CHAT_KEY_GROUP_CONTACTS "Chat"
+
+#define CHAT_GEOMETRY_FORMAT    "%d,%d,%d,%d"
 
 struct _GossipChatPriv {
 	GossipChatWindow *window;
@@ -511,6 +522,21 @@ chat_text_populate_popup_cb (GtkTextView *view,
 	gtk_widget_show (item);
 }
 
+static void     
+chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem, 
+				  GossipChatSpell *chat_spell)
+{
+	GossipChatPriv *priv;
+
+	priv = GET_PRIV (chat_spell->chat);
+
+	gossip_spell_dialog_show (chat_spell->chat,
+				  priv->spell,
+				  chat_spell->start,
+				  chat_spell->end,
+				  chat_spell->word);
+}
+
 GossipChatSpell *
 chat_spell_new (GossipChat  *chat, 
 		const gchar *word,
@@ -543,21 +569,6 @@ chat_spell_free (GossipChatSpell *chat_spell)
 	g_free (chat_spell);
 }
 
-static void     
-chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem, 
-				  GossipChatSpell *chat_spell)
-{
-	GossipChatPriv *priv;
-
-	priv = GET_PRIV (chat_spell->chat);
-
-	gossip_spell_dialog_show (chat_spell->chat,
-				  priv->spell,
-				  chat_spell->start,
-				  chat_spell->end,
-				  chat_spell->word);
-}
-
 gboolean
 gossip_chat_get_is_command (const gchar *str)
 {
@@ -657,16 +668,16 @@ gossip_chat_get_own_contact (GossipChat *chat)
 	return NULL;
 }
 
-void
-gossip_chat_get_geometry (GossipChat *chat,
-		          int        *width,
-			  int        *height)
+GossipChatroom *
+gossip_chat_get_chatroom (GossipChat *chat)
 {
-	g_return_if_fail (GOSSIP_IS_CHAT (chat));
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), NULL);
 
-	if (GOSSIP_CHAT_GET_CLASS (chat)->get_geometry) {
-		GOSSIP_CHAT_GET_CLASS (chat)->get_geometry (chat, width, height);
+	if (GOSSIP_CHAT_GET_CLASS (chat)->get_chatroom) {
+		return GOSSIP_CHAT_GET_CLASS (chat)->get_chatroom (chat);
 	}
+
+	return NULL;
 }
 
 GtkWidget *
@@ -682,12 +693,12 @@ gossip_chat_get_widget (GossipChat *chat)
 }
 
 gboolean
-gossip_chat_get_group_chat (GossipChat *chat)
+gossip_chat_is_group_chat (GossipChat *chat)
 {
 	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), FALSE);
 
-	if (GOSSIP_CHAT_GET_CLASS (chat)->get_group_chat) {
-		return GOSSIP_CHAT_GET_CLASS (chat)->get_group_chat (chat);
+	if (GOSSIP_CHAT_GET_CLASS (chat)->is_group_chat) {
+		return GOSSIP_CHAT_GET_CLASS (chat)->is_group_chat (chat);
 	}
 
 	return FALSE;
@@ -716,6 +727,26 @@ gossip_chat_set_show_contacts (GossipChat *chat,
 	}
 }
 
+void 
+gossip_chat_save_geometry (GossipChat *chat, 
+			   gint        x,
+			   gint        y,
+			   gint        w,
+			   gint        h)
+{
+	gossip_geometry_save_for_chat (chat, x, y, w, h);
+}
+
+void 
+gossip_chat_load_geometry (GossipChat *chat,
+			   gint       *x,
+			   gint       *y,
+			   gint       *w,
+			   gint       *h)
+{
+	gossip_geometry_load_for_chat (chat, x, y, w, h);
+}
+
 void
 gossip_chat_clear (GossipChat *chat)
 {
@@ -818,7 +849,7 @@ gossip_chat_present (GossipChat *chat)
 		GossipChatWindow *window;
 		gboolean          for_group_chat;
 
-		for_group_chat = gossip_chat_get_group_chat (chat);
+		for_group_chat = gossip_chat_is_group_chat (chat);
 
 		/* get the default window for either group chats or
 		   normal chats, we do not want to mix them */
