@@ -302,7 +302,7 @@ chatrooms_window_model_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 	const gchar          *name;
 	GossipChatroom       *chatroom;
 	GossipChatroomStatus  status;
- 	gchar                *status_str;
+ 	const gchar          *status_str;
 	gboolean              selected = FALSE;
 
 	attr_color = NULL;
@@ -321,13 +321,9 @@ chatrooms_window_model_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 	}
 
 	if (status == GOSSIP_CHATROOM_STATUS_ERROR) {
-		status_str = g_strdup (last_error);
+		status_str = last_error;
 	} else {
-/* 		status_str = gossip_chatroom_get_status_as_str (status); */
-
-		status_str = g_strdup_printf ("%s@%s", 
-					      gossip_chatroom_get_room (chatroom),
-					      gossip_chatroom_get_server (chatroom));
+		status_str = gossip_chatroom_get_id_str (chatroom);
 	}
 
 	str = g_strdup_printf ("%s\n%s", 
@@ -371,7 +367,6 @@ chatrooms_window_model_text_cell_data_func (GtkTreeViewColumn     *tree_column,
 	pango_attr_list_unref (attr_list);
 
 	g_free (str);
- 	g_free (status_str); 
 }
 
 static GList *
@@ -708,10 +703,10 @@ chatrooms_window_update_buttons (GossipChatroomsWindow *window)
 	switch (status) {
 	case GOSSIP_CHATROOM_STATUS_JOINING:
 		gtk_button_set_use_stock (button, TRUE);
-		gtk_button_set_label (button, GTK_STOCK_CANCEL);
+		gtk_button_set_label (button, GTK_STOCK_STOP);
 		
 		gtk_image_set_from_stock (GTK_IMAGE (image), 
-					  GTK_STOCK_CANCEL,
+					  GTK_STOCK_STOP,
 					  GTK_ICON_SIZE_BUTTON);
 		break;
 	case GOSSIP_CHATROOM_STATUS_ACTIVE:
@@ -774,7 +769,7 @@ chatrooms_window_join_cb (GossipChatroomProvider   *provider,
 	
 	if (result == GOSSIP_CHATROOM_JOIN_OK ||
 	    result == GOSSIP_CHATROOM_JOIN_ALREADY_OPEN) {
-		gossip_group_chat_show (provider, id);
+		gossip_group_chat_new (provider, id);
 	} 
 
 	chatrooms_window_update_buttons (window);
@@ -785,6 +780,10 @@ chatrooms_window_join_cb (GossipChatroomProvider   *provider,
 				chatrooms_window_chatroom_any_joining_foreach,
 				&any_joining);
 
+	if (result == GOSSIP_CHATROOM_JOIN_CANCELED) {
+		return;
+	}
+
 	if (!any_joining) {
 		gtk_widget_hide (window->window);
 	}
