@@ -29,6 +29,7 @@
 #include <libgossip/gossip-session.h>
 
 #include "gossip-app.h"
+#include "gossip-avatar-image.h"
 #include "gossip-contact-info-dialog.h"
 
 typedef struct {
@@ -38,12 +39,11 @@ typedef struct {
 
 	GtkWidget     *id_label;
 	GtkWidget     *name_label;
-	GtkWidget     *stub_id_label;
-	GtkWidget     *stub_name_label;
 	GtkWidget     *stub_email_label;
 	GtkWidget     *stub_web_label;
 	GtkWidget     *personal_status_label;
 	GtkWidget     *personal_status_hbox;
+	GtkWidget     *personal_vbox;
 	GtkWidget     *personal_table;
 
 	GtkWidget     *client_label;
@@ -66,6 +66,7 @@ typedef struct {
 
 	GtkWidget     *description_vbox;
 	GtkWidget     *description_label;
+	GtkWidget     *avatar_image;
 
 	gulong         contact_signal_handler;
 	gulong         presence_signal_handler;
@@ -221,12 +222,14 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 				  0, 0);
 		
 		widget = gtk_label_new (status);
+		gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
 				  widget, 
 				  1, 2,
 				  i, i + 1, 
-				  GTK_FILL, 0,
+				  GTK_FILL, 
+				  GTK_FILL,
 				  0, 0);
 
 		if (!resource) {
@@ -242,12 +245,14 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 				  0, 0.5);
 
 		widget = gtk_label_new (resource);
+		gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
 				  widget, 
 				  3, 4,
 				  i, i + 1,
-				  GTK_EXPAND | GTK_FILL, 0,
+				  GTK_EXPAND | GTK_FILL, 
+				  GTK_EXPAND | GTK_FILL,
 				  0, 0);
 	}
 
@@ -262,6 +267,7 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	GossipContactInfoDialog *dialog;
 	gboolean                 show_personal = FALSE;
 	const gchar             *str;
+	GdkPixbuf               *pixbuf;
 	
 	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
 	g_object_unref (contact);
@@ -273,11 +279,18 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	if (result != GOSSIP_RESULT_OK) {
 		if (dialog->dialog) {
 			gtk_widget_hide (dialog->personal_status_hbox);
+			gtk_widget_hide (dialog->personal_vbox);
 		}
 
 		return;
 	}
-
+	
+	pixbuf = gossip_pixbuf_avatar_from_vcard (vcard);
+	if (pixbuf != NULL) {
+		gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (dialog->avatar_image), pixbuf);
+		g_object_unref (pixbuf);
+	}
+	
 	str = gossip_vcard_get_description (vcard);
 	if (str && strlen (str) > 0) {
 		gtk_label_set_text (GTK_LABEL (dialog->description_label), str);
@@ -289,12 +302,6 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	str = gossip_vcard_get_name (vcard);
 	if (str && strlen (str) > 0) {
 		gtk_label_set_text (GTK_LABEL (dialog->name_label), str);
-
-		gtk_label_set_text (GTK_LABEL (dialog->stub_name_label),
-				    _("Name:"));
-	} else {
-		gtk_label_set_text (GTK_LABEL (dialog->stub_name_label),
-				    _("Alias:"));
 	}
 	
 	str = gossip_vcard_get_email (vcard);
@@ -350,16 +357,12 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		gtk_widget_hide (dialog->stub_web_label);
 	}
 
+	gtk_widget_hide (dialog->personal_status_hbox);
+
 	if (show_personal) {
-		GtkSizeGroup *size_group;
-
-		gtk_widget_hide (dialog->personal_status_hbox);
-		gtk_widget_show (dialog->personal_table);
-
-		size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-		gtk_size_group_add_widget (size_group, dialog->stub_id_label);
-		gtk_size_group_add_widget (size_group, dialog->stub_email_label);
-		g_object_unref (size_group);
+		gtk_widget_show (dialog->personal_vbox);
+	} else {
+		gtk_widget_hide (dialog->personal_vbox);
 	}
 }
 
@@ -425,10 +428,11 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 		gtk_widget_show (dialog->client_table);
 
 		size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-		gtk_size_group_add_widget (size_group, dialog->stub_id_label);
 		gtk_size_group_add_widget (size_group, dialog->stub_client_label);
 		g_object_unref (size_group);
-	} 
+	} else {
+		gtk_widget_hide (dialog->client_vbox);
+	}
 }
 
 static void
@@ -530,6 +534,8 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	gchar                   *str, *tmp_str;
 	GtkSizeGroup            *size_group;
 	guint                    id;
+	GtkWidget               *avatar_image_placeholder;
+	GdkPixbuf               *pixbuf;
 
 	contact_info_dialog_init ();
 	
@@ -557,12 +563,11 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 				       "client_label", &dialog->client_label,
 				       "version_label", &dialog->version_label,
 				       "os_label", &dialog->os_label,
+				       "personal_vbox", &dialog->personal_vbox,
 				       "personal_table", &dialog->personal_table,
 				       "description_vbox", &dialog->description_vbox,
 				       "description_label", &dialog->description_label,
 				       "client_table", &dialog->client_table,
-				       "stub_id_label", &dialog->stub_id_label,
-				       "stub_name_label", &dialog->stub_name_label,
 				       "stub_email_label", &dialog->stub_email_label,
 				       "stub_web_label", &dialog->stub_web_label,
 				       "stub_client_label", &dialog->stub_client_label,
@@ -577,8 +582,21 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 				       "subscription_label", &dialog->subscription_label,
 				       "presence_list_vbox", &dialog->presence_list_vbox,
 				       "presence_table", &dialog->presence_table,
+				       "avatar_image_placeholder", &avatar_image_placeholder,
 				       NULL);
+	
+	dialog->avatar_image = gossip_avatar_image_new (NULL);
+	gtk_container_add (GTK_CONTAINER (avatar_image_placeholder),
+			   dialog->avatar_image);
 
+	pixbuf = gdk_pixbuf_new_from_file (IMAGEDIR "/vcard_48.png", NULL);
+	if (pixbuf) {
+		gossip_avatar_image_set_pixbuf (
+			GOSSIP_AVATAR_IMAGE (dialog->avatar_image), pixbuf);
+		g_object_unref (pixbuf);
+	}
+	gtk_widget_show (dialog->avatar_image);
+	
 	gossip_glade_connect (glade,
 			      dialog,
 			      "contact_information_dialog", "destroy", contact_info_dialog_destroy_cb,
@@ -591,7 +609,6 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	/* Align widgets */
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
-	gtk_size_group_add_widget (size_group, dialog->stub_name_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_email_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_web_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_client_label);
@@ -610,8 +627,6 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	gtk_label_set_text (GTK_LABEL (dialog->id_label), 
 			    gossip_contact_get_id (contact));
 
-	gtk_label_set_text (GTK_LABEL (dialog->stub_name_label),
-			    _("Alias:"));
 	gtk_label_set_text (GTK_LABEL (dialog->name_label), 
 			    gossip_contact_get_name (contact));
 
@@ -635,11 +650,14 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 
 	/* get vcard and version info */
 	str = g_strdup_printf ("<i>%s</i>", 
-			       _("Requested Information"));
+			       _("Information requested..."));
 	gtk_label_set_markup (GTK_LABEL (dialog->personal_status_label), str);
 	gtk_label_set_markup (GTK_LABEL (dialog->client_status_label), str);
 	g_free (str);
 
+	gtk_widget_show (dialog->personal_vbox);
+	gtk_widget_show (dialog->client_vbox);
+
 	gossip_session_get_vcard (session,
 				  account,
 				  contact,
