@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2004 Imendio AB
+ * Copyright (C) 2003-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,40 +33,23 @@
 #include "gossip-spell.h"
 
 typedef struct {
-	GossipApp *app;
- 	
 	GtkWidget *dialog;
 	GtkWidget *notebook;
+
 	GtkWidget *sound_checkbutton;
 	GtkWidget *silent_busy_checkbutton;
 	GtkWidget *silent_away_checkbutton;
-	GtkWidget *smileys_checkbutton;
         GtkWidget *separate_windows_checkbutton;
-	GtkWidget *compact_checkbutton;
-	GtkWidget *leaving_entry;
-	GtkWidget *away_entry;
+
+	GtkWidget *smileys_checkbutton;
+	GtkWidget *theme_combobox;
+
 	GtkWidget *spell_checker_vbox;
-	GtkWidget *spell_checker_checkbutton;
 	GtkWidget *spell_checker_treeview;
 
-	GtkWidget *theme_combobox;
-	
-	GList     *ids;
+	GList     *gconf_ids;
 } GossipPreferences;
 
-enum {
-	COL_LANG_ENABLED,
-	COL_LANG_CODE,
-	COL_LANG_NAME,
-	COL_LANG_COUNT
-};
-
-enum {
-	COL_COMBO_VISIBLE_NAME,
-	COL_COMBO_NAME,
-	COL_COMBO_COUNT
-};
-
 static void     preferences_setup_widgets                (GossipPreferences      *preferences);
 static void     preferences_languages_setup              (GossipPreferences      *preferences);
 static void     preferences_languages_add                (GossipPreferences      *preferences);
@@ -93,23 +76,23 @@ static void     preferences_set_string_from_gconf        (const gchar
 static void     preferences_set_string_combo_from_gconf  (const gchar            *key,
 							  GtkWidget              *widget);
 static void     preferences_notify_int_cb                (GConfClient            *client,
-							  guint                   cnxn_id,
+							  guint                   id,
 							  GConfEntry             *entry,
 							  gpointer                user_data);
 static void     preferences_notify_string_cb             (GConfClient            *client,
-							  guint                   cnxn_id,
+							  guint                   id,
 							  GConfEntry             *entry,
 							  gpointer                user_data);
 static void     preferences_notify_string_combo_cb       (GConfClient            *client,
-							  guint                   cnxn_id,
+							  guint                   id,
 							  GConfEntry             *entry,
 							  gpointer                user_data);
 static void     preferences_notify_bool_cb               (GConfClient            *client,
-							  guint                   cnxn_id,
+							  guint                   id,
 							  GConfEntry             *entry,
 							  gpointer                user_data);
 static void     preferences_notify_sensitivity_cb        (GConfClient            *client,
-							  guint                   cnxn_id,
+							  guint                   id,
 							  GConfEntry             *entry,
 							  gpointer                user_data);
 static void     preferences_hookup_spin_button           (GossipPreferences      *preferences,
@@ -141,6 +124,19 @@ static void     preferences_response_cb                  (GtkWidget
 							  gint                    response,
 							  GossipPreferences      *preferences);
 
+enum {
+	COL_LANG_ENABLED,
+	COL_LANG_CODE,
+	COL_LANG_NAME,
+	COL_LANG_COUNT
+};
+
+enum {
+	COL_COMBO_VISIBLE_NAME,
+	COL_COMBO_NAME,
+	COL_COMBO_COUNT
+};
+
 static void
 preferences_setup_widgets (GossipPreferences *preferences)
 {
@@ -160,17 +156,13 @@ preferences_setup_widgets (GossipPreferences *preferences)
 					GCONF_PATH "/sound/play_sounds",
 					preferences->silent_busy_checkbutton);
 
-        preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/conversation/graphical_smileys",
-					  preferences->smileys_checkbutton);
-
         preferences_hookup_toggle_button (preferences,
 					  GCONF_PATH "/ui/separate_chat_windows",
 					  preferences->separate_windows_checkbutton);
 
-	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/conversation/enable_spell_checker",
-					  preferences->spell_checker_checkbutton);
+        preferences_hookup_toggle_button (preferences,
+					  GCONF_PATH "/conversation/graphical_smileys",
+					  preferences->smileys_checkbutton);
 
 	preferences_hookup_string_combo (preferences,
 					 GCONF_PATH "/conversation/theme",
@@ -288,7 +280,7 @@ preferences_languages_save (GossipPreferences *preferences)
 				&languages);
 
 	if (!languages) {
-		/* default to english */
+		/* Default to english */
 		languages = g_strdup ("en");
 	}
 	
@@ -382,7 +374,6 @@ preferences_languages_load_foreach (GtkTreeModel  *model,
 		return TRUE;
 	}
 
-
 	gtk_tree_model_get (model, iter, COL_LANG_CODE, &code, -1);
 	if (!code) {
 		return FALSE;
@@ -533,9 +524,9 @@ preferences_set_string_combo_from_gconf (const gchar *key, GtkWidget *widget)
 
 static void
 preferences_notify_int_cb (GConfClient *client,
-			   guint        cnxn_id,
+			   guint        id,
 			   GConfEntry  *entry,
-			      gpointer   user_data)
+			   gpointer   user_data)
 {
 	GConfValue *value;
 
@@ -547,9 +538,9 @@ preferences_notify_int_cb (GConfClient *client,
 
 static void
 preferences_notify_string_cb (GConfClient *client,
-		 guint        cnxn_id,
-		 GConfEntry  *entry,
-		 gpointer     user_data)
+			      guint        id,
+			      GConfEntry  *entry,
+			      gpointer     user_data)
 {
 	GConfValue *value;
 	
@@ -561,7 +552,7 @@ preferences_notify_string_cb (GConfClient *client,
 
 static void
 preferences_notify_string_combo_cb (GConfClient *client,
-				    guint        cnxn_id,
+				    guint        id,
 				    GConfEntry  *entry,
 				    gpointer     user_data)
 {
@@ -570,7 +561,7 @@ preferences_notify_string_combo_cb (GConfClient *client,
 
 static void
 preferences_notify_bool_cb (GConfClient *client,
-			    guint        cnxn_id,
+			    guint        id,
 			    GConfEntry  *entry,
 			    gpointer     user_data)
 {
@@ -584,7 +575,7 @@ preferences_notify_bool_cb (GConfClient *client,
 
 static void
 preferences_notify_sensitivity_cb (GConfClient *client,
-				   guint        cnxn_id,
+				   guint        id,
 				   GConfEntry  *entry,
 				   gpointer     user_data)
 {
@@ -598,8 +589,8 @@ preferences_notify_sensitivity_cb (GConfClient *client,
 
 static void
 preferences_hookup_spin_button (GossipPreferences *preferences,
-		    const gchar       *key,
-		    GtkWidget         *widget)
+				const gchar       *key,
+				GtkWidget         *widget)
 {
 	guint id;
 
@@ -625,14 +616,14 @@ preferences_hookup_spin_button (GossipPreferences *preferences,
 				      NULL,
 				      NULL);
 	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+	preferences->gconf_ids = g_list_prepend (preferences->gconf_ids,
+						 GUINT_TO_POINTER (id));
 }
 
 static void
 preferences_hookup_entry (GossipPreferences *preferences,
-	      const gchar       *key,
-	      GtkWidget         *widget)
+			  const gchar       *key,
+			  GtkWidget         *widget)
 {
 	guint id;
 
@@ -657,14 +648,14 @@ preferences_hookup_entry (GossipPreferences *preferences,
 				      NULL,
 				      NULL);
 	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+	preferences->gconf_ids = g_list_prepend (preferences->gconf_ids,
+						 GUINT_TO_POINTER (id));
 }
 
 static void
 preferences_hookup_toggle_button (GossipPreferences *preferences,
-		      const gchar       *key,
-		      GtkWidget         *widget)
+				  const gchar       *key,
+				  GtkWidget         *widget)
 {
 	guint id;
 	
@@ -685,8 +676,8 @@ preferences_hookup_toggle_button (GossipPreferences *preferences,
 				      NULL,
 				      NULL);
 	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+	preferences->gconf_ids = g_list_prepend (preferences->gconf_ids,
+						 GUINT_TO_POINTER (id));
 }
 
 static void
@@ -713,14 +704,14 @@ preferences_hookup_string_combo (GossipPreferences *preferences,
 				      NULL,
 				      NULL);
 	
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+	preferences->gconf_ids = g_list_prepend (preferences->gconf_ids,
+						 GUINT_TO_POINTER (id));
 }
 
 static void
 preferences_hookup_sensitivity (GossipPreferences *preferences,
-			       const gchar       *key,
-			       GtkWidget         *widget)
+				const gchar       *key,
+				GtkWidget         *widget)
 {
 	gboolean value;
 	guint    id;
@@ -735,8 +726,8 @@ preferences_hookup_sensitivity (GossipPreferences *preferences,
 				      NULL,
 				      NULL);
 
-	preferences->ids = g_list_prepend (preferences->ids,
-					   GUINT_TO_POINTER (id));
+	preferences->gconf_ids = g_list_prepend (preferences->gconf_ids,
+						 GUINT_TO_POINTER (id));
 }
 
 static void
@@ -822,13 +813,14 @@ preferences_destroy_cb (GtkWidget         *widget,
 {
 	GList *l;
 
-	for (l = preferences->ids; l; l = l->next) {
-		guint id = GPOINTER_TO_UINT (l->data);
+	for (l = preferences->gconf_ids; l; l = l->next) {
+		guint id;
 
+		id = GPOINTER_TO_UINT (l->data);
 		gconf_client_notify_remove (gossip_app_get_gconf_client (), id);
 	}
 
-	g_list_free (preferences->ids);
+	g_list_free (preferences->gconf_ids);
 	g_free (preferences);
 }
 
@@ -844,7 +836,6 @@ gossip_preferences_show (void)
 	}
 	
         preferences = g_new0 (GossipPreferences, 1);
-	preferences->app = gossip_app_get ();
 
 	glade = gossip_glade_get_file (
 		GLADEDIR "/main.glade",
@@ -855,16 +846,20 @@ gossip_preferences_show (void)
 		"sound_checkbutton", &preferences->sound_checkbutton,
 		"silent_busy_checkbutton", &preferences->silent_busy_checkbutton,
 		"silent_away_checkbutton", &preferences->silent_away_checkbutton,
-		"smileys_checkbutton", &preferences->smileys_checkbutton,
 		"separate_windows_checkbutton", &preferences->separate_windows_checkbutton,
-		"spell_checker_vbox", &preferences->spell_checker_vbox,
-		"spell_checker_checkbutton", &preferences->spell_checker_checkbutton,
-		"spell_checker_treeview", &preferences->spell_checker_treeview,
+		"smileys_checkbutton", &preferences->smileys_checkbutton,
 		"theme_combobox", &preferences->theme_combobox,
+		"spell_checker_vbox", &preferences->spell_checker_vbox,
+		"spell_checker_treeview", &preferences->spell_checker_treeview,
 		NULL);
 
-	g_object_add_weak_pointer (G_OBJECT (preferences->dialog),
-				   (gpointer) &preferences);
+	gossip_glade_connect (glade, 
+			      preferences,
+			      "preferences_dialog", "destroy", preferences_destroy_cb,
+			      "preferences_dialog", "response", preferences_response_cb,
+			      NULL);
+
+	g_object_add_weak_pointer (G_OBJECT (preferences->dialog), (gpointer) &preferences);
 	
 	preferences_themes_setup (preferences);
 
@@ -874,21 +869,15 @@ gossip_preferences_show (void)
 	preferences_languages_add (preferences);
 	preferences_languages_load (preferences);
 	
-	gossip_glade_connect (glade, 
-			      preferences,
-			      "preferences_dialog", "destroy", preferences_destroy_cb,
-			      "preferences_dialog", "response", preferences_response_cb,
-			      NULL);
-	
 	if (gossip_spell_supported ()) {
-		GtkWidget *spell_checking_page;
+		GtkWidget *page;
 
-		spell_checking_page = gtk_notebook_get_nth_page (
-			GTK_NOTEBOOK (preferences->notebook), 1);
-		gtk_widget_show (spell_checking_page);
+		page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (preferences->notebook), 2);
+		gtk_widget_show (page);
 	}
 
 	gtk_window_set_transient_for (GTK_WINDOW (preferences->dialog),
 				      GTK_WINDOW (gossip_app_get_window ()));
+
 	gtk_widget_show (preferences->dialog);
 }
