@@ -108,9 +108,6 @@ static gboolean       private_chat_delete_event_cb              (GtkWidget
 static const gchar *  private_chat_get_name                     (GossipChat             *chat);
 static gchar *        private_chat_get_tooltip                  (GossipChat             *chat);
 static GdkPixbuf *    private_chat_get_status_pixbuf            (GossipChat             *chat);
-static void           private_chat_get_geometry                 (GossipChat             *chat,
-								 int                    *width,
-								 int                    *height);
 static GtkWidget *    private_chat_get_widget                   (GossipChat             *chat);
 static GossipContact *private_chat_get_contact                  (GossipChat             *chat);
 static GossipContact *private_chat_get_own_contact              (GossipChat             *chat);
@@ -137,7 +134,6 @@ gossip_private_chat_class_init (GossipPrivateChatClass *klass)
 	chat_class->get_status_pixbuf = private_chat_get_status_pixbuf;
 	chat_class->get_contact       = private_chat_get_contact;
 	chat_class->get_own_contact   = private_chat_get_own_contact;
-	chat_class->get_geometry      = private_chat_get_geometry;
 	chat_class->get_widget        = private_chat_get_widget;
 }
 
@@ -248,9 +244,11 @@ private_chat_create_gui (GossipPrivateChat *chat)
 static void
 private_chat_update_locked_resource (GossipPrivateChat *chat)
 {
-	GossipPrivateChatPriv *priv = chat->priv;
+	GossipPrivateChatPriv *priv;
 	const gchar           *roster_resource;
 
+	priv = chat->priv;
+
 	if (!gossip_contact_is_online (priv->contact)) {
 		g_free (priv->roster_resource);
 		priv->roster_resource = NULL;
@@ -263,6 +261,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 
 	roster_resource = gossip_session_get_active_resource (gossip_app_get_session (), 
 							      priv->contact);
+
 	/* It seems like some agents don't set a resource sometimes (ICQ for
 	 * example). I don't know if it's a bug or not, but we need to handle
 	 * those cases either way.
@@ -305,8 +304,6 @@ private_chat_send (GossipPrivateChat *chat,
 
 	priv = chat->priv;
 
-	gossip_app_force_non_away ();
-
         if (msg == NULL || msg[0] == '\0') {
                 return;
         }
@@ -316,6 +313,8 @@ private_chat_send (GossipPrivateChat *chat,
                 return;
         }
 
+	gossip_app_force_non_away ();
+
         private_chat_composing_remove_timeout (chat);
 
 	private_chat_update_locked_resource (chat);
@@ -744,15 +743,6 @@ private_chat_get_own_contact (GossipChat *chat)
 	return priv->own_contact;
 }
 
-static void
-private_chat_get_geometry (GossipChat *chat,
-		           int        *width,
-			   int        *height)
-{
-	*width  = 350;
-	*height = 250;
-}
-
 static GtkWidget *
 private_chat_get_widget (GossipChat *chat)
 {
@@ -864,7 +854,11 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 	if (resource) {
 		gboolean is_different;
 
-		is_different = g_ascii_strcasecmp (resource, priv->locked_resource) != 0;
+		if (priv->locked_resource) {
+			is_different = g_ascii_strcasecmp (resource, priv->locked_resource) != 0;
+		} else {
+			is_different = TRUE;
+		}
 
 		if (!priv->locked_resource || is_different) {
 			g_free (priv->locked_resource);
