@@ -28,13 +28,16 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
+#include "libgossip/gossip-utils.h"
 #include "gossip-status-presets.h"
 
+#define DEBUG_MSG(x)  
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+
 #define STATUS_PRESETS_XML_FILENAME "status-presets.xml"
 #define STATUS_PRESETS_DTD_FILENAME "gossip-status-presets.dtd"
 #define STATUS_PRESETS_MAX_EACH     15
 
-#define d(x) 
 
 typedef struct {
 	gchar               *status;
@@ -67,7 +70,7 @@ gossip_status_presets_get_all (void)
 
 	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
+		g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
 
 	file_with_path = g_build_filename (dir, STATUS_PRESETS_XML_FILENAME, NULL);
@@ -88,25 +91,25 @@ status_presets_file_parse (const gchar *filename)
 	xmlNodePtr        presets_node;
 	xmlNodePtr        node;
 	
-	d(g_print ("Attempting to parse file:'%s'...\n", filename));
+	DEBUG_MSG (("StatusPresets: Attempting to parse file:'%s'...", filename));
 
  	ctxt = xmlNewParserCtxt ();
 
 	/* Parse and validate the file. */
-	doc = xmlCtxtReadFile (ctxt, filename, NULL, XML_PARSE_DTDVALID);	
+	doc = xmlCtxtReadFile (ctxt, filename, NULL, 0);	
 	if (!doc) {
 		g_warning ("Failed to parse file:'%s'", filename);
 		xmlFreeParserCtxt (ctxt);
 		return;
 	}
 
-	if (!ctxt->valid) {
-		g_warning ("Failed to validate file:'%s'",  filename);
+	if (!gossip_xml_validate (doc, STATUS_PRESETS_DTD_FILENAME)) {
+		g_warning ("Failed to validate file:'%s'", filename);
 		xmlFreeDoc(doc);
 		xmlFreeParserCtxt (ctxt);
 		return;
 	}
-	
+
 	/* The root node, presets. */
 	presets_node = xmlDocGetRootElement (doc);
 
@@ -119,7 +122,7 @@ status_presets_file_parse (const gchar *filename)
 			StatusPreset        *preset;
 
 			status = (gchar *) xmlNodeGetContent (node);
-			state_str = (gchar *) xmlGetProp (node, BAD_CAST ("presence"));
+			state_str = (gchar *) xmlGetProp (node, "presence");
 
 			if (state_str) {
 				if (strcmp (state_str, "available") == 0) {
@@ -148,7 +151,7 @@ status_presets_file_parse (const gchar *filename)
 		node = node->next;
 	}
 	
-	d(g_print ("Parsed %d status presets\n", g_list_length (presets)));
+	DEBUG_MSG (("StatusPresets: Parsed %d status presets", g_list_length (presets)));
 
 	xmlFreeDoc (doc);
 	xmlFreeParserCtxt (ctxt);
@@ -183,25 +186,25 @@ status_presets_file_save (void)
 	xmlNodePtr  root;
 	GList      *l;
 	gchar      *dtd_file;
-	gchar      *xml_dir;
-	gchar      *xml_file;
+	gchar      *dir;
+	gchar      *file;
 	gint        count[4] = { 0, 0, 0, 0};
 
-	xml_dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
-	if (!g_file_test (xml_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		mkdir (xml_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
+	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+		g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
 
-	xml_file = g_build_filename (xml_dir, STATUS_PRESETS_XML_FILENAME, NULL);
-	g_free (xml_dir);
+	file = g_build_filename (dir, STATUS_PRESETS_XML_FILENAME, NULL);
+	g_free (dir);
 
 	dtd_file = g_build_filename (DTDDIR, STATUS_PRESETS_DTD_FILENAME, NULL);
 
-	doc = xmlNewDoc (BAD_CAST "1.0");
-	root = xmlNewNode (NULL, BAD_CAST "presets");
+	doc = xmlNewDoc ("1.0");
+	root = xmlNewNode (NULL, "presets");
 	xmlDocSetRootElement (doc, root);
 
-	dtd = xmlCreateIntSubset (doc, BAD_CAST "presets", NULL, BAD_CAST dtd_file);
+	dtd = xmlCreateIntSubset (doc, "presets", NULL, dtd_file);
 
 	for (l = presets; l; l = l->next) {
 		StatusPreset *sp;
@@ -212,16 +215,16 @@ status_presets_file_save (void)
 
 		switch (sp->state) {
 		case GOSSIP_PRESENCE_STATE_AVAILABLE:
-			state = BAD_CAST "available";
+			state = "available";
 			break;
 		case GOSSIP_PRESENCE_STATE_BUSY:
-			state = BAD_CAST "busy";
+			state = "busy";
 			break;
 		case GOSSIP_PRESENCE_STATE_AWAY:
-			state = BAD_CAST "away";
+			state = "away";
 			break;
 		case GOSSIP_PRESENCE_STATE_EXT_AWAY:
-			state = BAD_CAST "ext_away";
+			state = "ext_away";
 			break;
 		default:
 			continue;
@@ -232,18 +235,18 @@ status_presets_file_save (void)
 			continue;
 		}
 		
-		subnode = xmlNewChild (root,
-				       NULL,
-				       BAD_CAST "status",
-				       BAD_CAST sp->status);
-		xmlNewProp (subnode, BAD_CAST "presence", state);	
+		subnode = xmlNewTextChild (root,
+					   NULL,
+					   "status",
+					   sp->status);
+		xmlNewProp (subnode, "presence", state);	
 	}
 
-	d(g_print ("Saving file:'%s'\n", xml_file));
-	xmlSaveFormatFileEnc (xml_file, doc, "utf-8", 1);
+	DEBUG_MSG (("StatusPresets: Saving file:'%s'", file));
+	xmlSaveFormatFileEnc (file, doc, "utf-8", 1);
 	xmlFreeDoc (doc);
 
-	g_free (xml_file);
+	g_free (file);
 
 	return TRUE;
 }
