@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -178,13 +178,13 @@ subscription_dialog_setup_groups (GtkComboBoxEntry *comboboxentry)
 	GtkCellRenderer *renderer;
 	GtkTreeIter      iter;	
 	GList           *l;
-	GList           *groups;
+	GList           *all_groups;
 
 	store = gtk_list_store_new (1, G_TYPE_STRING);
 
-	groups = gossip_session_get_groups (gossip_app_get_session ());
+	all_groups = gossip_session_get_groups (gossip_app_get_session ());
 
-	for (l = groups; l; l = l->next) {
+	for (l = all_groups; l; l = l->next) {
 		const gchar *group;
 		
 		group = l->data;
@@ -210,6 +210,8 @@ subscription_dialog_setup_groups (GtkComboBoxEntry *comboboxentry)
 					NULL);
 
 	g_object_unref (store);
+
+	g_list_free (all_groups);
 }
 
 static void
@@ -294,18 +296,20 @@ subscription_dialog_vcard_cb (GossipResult              result,
 	gtk_label_set_text (GTK_LABEL (dialog->id_label_value), 
 			    gossip_contact_get_id (dialog->contact));
 
-	/* set up url button */
+	/* Set up url button */
 	if (url && strlen (url) > 0) {
 		GArray *start, *end;
 
 		start = g_array_new (FALSE, FALSE, sizeof (gint));
 		end = g_array_new (FALSE, FALSE, sizeof (gint));
 		
-		num_matches = gossip_utils_regex_match (GOSSIP_REGEX_ALL, 
-							url, start, end);
+		num_matches = gossip_regex_match (GOSSIP_REGEX_ALL, url, start, end);
+
+		g_array_free (start, TRUE);
+		g_array_free (end, TRUE);
 	}
 
-	/* add url button to table */
+	/* Add url button to table */
 	if (num_matches > 0) {
 		GtkWidget *href;
 		GtkWidget *alignment;
